%*************************************************************************************************************;
%*                                                                                                           *;
%* Article: SAS macros for calculation of Population Attributable Fraction (PAF) in a cohort study design    *;
%* Authors: Maarit A. Laaksonen, Esa Virtala, Paul Knekt, Hannu Oja, and Tommi Hrknen                      *;
%*                                                                                                           *;
%*                                                                                                           *;
%* EST_MATRIX                                                                                                *;
%*                                                                                                           *;
%* Preparation of design matrices, parameter estimates and their covariances needed for the estimation       *;
%* of Population Attributable Fraction (PAF) and its confidence interval.                                    *;
%*                                                                                                           *;
%* INPUT:                                                                                                    *;
%*                                                                                                           *;
%*   DATASET                = Input data matrix.                                                             *;
%*                                                                                                           *;
%*   CENSORVARIABLE         = Variable which indicates whether censoring before the outcome of interest      *;
%*                            occurred. A binary variable (0/1).                                             *;
%*                                                                                                           *;
%*   CENSORVALUE            = Value (0/1) which indicates that censoring before the outcome of interest      *;
%*                            occurred.                                                                      *;
%*                                                                                                           *;
%*   TIMEVARIABLE           = Variable which indicates the individual length of follow-up within each        *;
%*                            follow-up time interval.                                                       *;
%*                                                                                                           *;
%*   IDVARIABLE             = Individual identification variable.                                            *;
%*                                                                                                           *;
%*   COHORTVARIABLE         = Categorical variable which indicates to which birth cohort the individual      *;
%*                            belongs to. Optional: if not given, the baseline hazard in the piecewise       *;
%*                            constant hazard model is only stratified according to the follow-up time       *;
%*                            interval.                                                                      *;
%*                                                                                                           *;
%*   PERIODVARIABLE         = Categorical variable which indicates the cumulative follow-up time by the      *;
%*                            beginning of each follow-up time interval.                                     *;
%*                                                                                                           *;
%*   PERIODLENGTH           = Length of follow-up intervals.                                                 *;
%*                                                                                                           *;
%*   PERIODCOUNT            = Number of follow-up intervals.                                                 *;
%*                                                                                                           *;
%*   DELTALENGTH            = Length of follow-up.                                                           *;
%*                                                                                                           *;
%*   CLASSVARIABLES         = Categorical variables included in the model (separated by blanks).             *;
%*                                                                                                           *;
%*   CLASSORDER             = Order of the categories of the variables in the lifereg analysis (affects      *;
%*                            the reference category): DESC=descending / ASC=ascending.                      *;
%*                                                                                                           *;
%*   COVARIATE_MODEL        = All the variables (categorical, continuous, or their interactions) included    *;
%*                            in the model (separated by blanks, interactions denoted by asterisk between    *;
%*                            the variables, VARIABLE1*VARIABLE2). Corresponds to the normal model statement *;
%*                            syntax of SAS.                                                                 *;
%*                                                                                                           *;
%*   MODIFICATIONS          = Determines the reference category for the risk factors of interest to          *;
%*                            which all or some individuals are hypothetically moved in the calculation      *;
%*                            of PAF. If all the individuals from all the categories of the risk factor      *;
%*                            are moved to the reference category, the modification statement is denoted     *;
%*                            by 'RISK FACTOR = REFERENCE CATEGORY'. If individuals from only some of        *;
%*                            the categories of the risk factor are moved to the reference category,         *;
%*                            then the modification statement is denoted using the IF-THEN statement:        *;
%*                            'IF RISK FACTOR IN(CATEGORIES THAT ARE MODIFIED SEPARATED BY COMMA)            *;
%*                             THEN RISK FACTOR = REFERENCE CATEGORY'. If several risk factors are           *;
%*                            modified simultaneously, they are separated by semicolons.                     *;
%*                                                                                                           *;
%*   PRINT                  = Defines what is printed out from the LIFEREG procedure: YES = all output /     *;
%*                            NO = only the relative risks (RR) and their 95% confidence intervals (CI)      *;
%*                           (formed with the help of the PROC LIFEREG output). Optional: if not given,      *;
%*                            only the RRs and their 95% CIs are printed out.                                *;
%*                                                                                                           *;
%* OUTPUT:                                                                                                   *;
%*                                                                                                           *;
%*   DESIGN_1               = Design matrix which indicates to which categories of the baseline hazard       *;
%*                            variables (follow-up time intervals, birth cohorts, and their                  *;
%*                            interactions) each individual belongs on each follow-up time interval and      *;
%*                            which values of the risk factors each individual has.                          *;
%*                                                                                                           *;
%*   DESIGN_2               = Design matrix which indicates to which categories of the baseline hazard       *;
%*                            variables (follow-up time intervals, birth cohorts, and their                  *;
%*                            interactions) each individual belongs on each follow-up time interval and      *;
%*                            which values of the risk factors each individual has after their               *;
%*                            hypothetical change to the chosen reference categories indicated in the        *;
%*                            MODIFICATIONS.                                                                 *;
%*                                                                                                           *;
%*  &CENSORVARIABLE._EST  = Vertical vector of parameter estimates for the baseline hazard variables         *;
%*                            and the risk factors, where &CENSORVARIABLE is substituted with the            *;
%*                            name of the censor variable given in the macro call.                           *;
%*                                                                                                           *;
%*  &CENSORVARIABLE._COVB = Covariance matrix of the parameter estimates for the baseline hazard             *;
%*                            variables and the risk factors, where &CENSORVARIABLE is substituted           *;
%*                            with the name of the censor variable given in the PAF macro call.              *;
%*                                                                                                           *;
%*  &CENSORVARIABLE._RR   = Relative risks and their confidence intervals for the variables given in the     *;
%*                            COVARIATE_MODEL.                                                               *;
%*                                                                                                           *;
%*  &CENSORVARIABLE._CONV = Convergence status of LIFEREG analysis.                                          *;
%*                                                                                                           *;
%*************************************************************************************************************;

%MACRO EST_MATRIX(
  DATASET         = ,
  CENSORVARIABLE  = ,
  CENSORVALUE     = 0,
  TIMEVARIABLE    = ,
  IDVARIABLE      = ,
  COHORTVARIABLE  = ,
  PERIODVARIABLE  = ,
  PERIODLENGTH    = ,
  PERIODCOUNT     = ,
  DELTALENGTH     = ,
  CLASSVARIABLES  = ,
  CLASSORDER      = ,
  COVARIATE_MODEL = ,
  MODIFICATIONS   = ,
  PRINT           =
 );
   ODS NORESULTS;
   OPTIONS NOSOURCE;

  %LET CENS=&CENSORVARIABLE;
  %IF %LENGTH(&CENS)>20 %THEN %LET CENS=%SUBSTR(&CENSORVARIABLE,1,20);
  %IF &CENSORVALUE=%STR( ) %THEN %LET CENSORVALUE=0;

  %IF %UPCASE(&PRINT)^=YES %THEN %LET PRINT=NO;

  %LET BASELINE_MODEL=&PERIODVARIABLE;
  %IF &COHORTVARIABLE^=%STR( ) %THEN %LET BASELINE_MODEL=%STR(&COHORTVARIABLE|&PERIODVARIABLE);

  %LET _BASELINE_MODELVARS_=&COHORTVARIABLE &PERIODVARIABLE;
  %LET _COVARIATE_MODELVARS_=%SYSFUNC(TRANSLATE(&COVARIATE_MODEL,' ','|',' ','*',' ','(',' ',')',' ',','));

  %LET CLASSORDER=%UPCASE(&CLASSORDER);
  %IF &CLASSORDER^=DESC %THEN %LET CLASSORDER=ASC;

  %*****************************************************************************************************;
  %*   REFINEMENT OF DATA                                                                              *;
  %*****************************************************************************************************;

  %* The input data matrix is refined to include only the variables specified in the macro call.       *;
  %* The individuals with missing values in some of the censor variables, birth cohort variable, or    *;
  %* covariates included in the model are removed from the data.                                       *;
  %* These steps are done in order to make the design matrix and the vector of parameter               *;
  %* estimates to match (see matrix _DATASET_ in Work library).                                        *;

  DATA _DATASET_;
    RETAIN _TMP_NUM_&SYSDATE9 0 _TMP_CHAR_&SYSDATE9 'X';
    SET &DATASET(KEEP=&IDVARIABLE &CENSORVARIABLE &TIMEVARIABLE
                      &CLASSVARIABLES
                      &_BASELINE_MODELVARS_
                      &_COVARIATE_MODELVARS_
                      );
    ARRAY NUM_MISS(*) _NUMERIC_;
    ARRAY CHR_MISS(*) _CHARACTER_;
    IF NMISS(OF NUM_MISS(*)) THEN DELETE;
    IF CMISS(OF CHR_MISS(*)) THEN DELETE;
    DROP _TMP_NUM_&SYSDATE9 _TMP_CHAR_&SYSDATE9;
  RUN;


  %* If given CLASSORDER=DESC then create formats to reverse the order of all CLASSVARIABLES.          *;

  %IF &CLASSORDER=DESC %THEN %DO;
      %LET _FMTVAR=%SYSFUNC(COMPBL(&_BASELINE_MODELVARS_ &CLASSVARIABLES));
      %LET _VARCNT=%SYSFUNC(COUNTC(&_FMTVAR,%STR( )));
      %LET _VARCNT=%EVAL(&_VARCNT+1);
      %DO _I=1 %TO &_VARCNT;
          %LET _V=%SCAN(&_FMTVAR,&_I,%STR( ));
          PROC SQL NOPRINT;
            CREATE TABLE _&_I AS SELECT DISTINCT &_V FROM _DATASET_ WHERE &_V IS NOT MISSING ORDER BY &_V;
            SELECT UPCASE(TYPE) INTO :_T FROM SASHELP.VCOLUMN WHERE LIBNAME="WORK" AND MEMNAME="_&_I";
          QUIT;
          DATA _NULL_;
            LENGTH _VALUE $200;
            SET _&_I END=EOF;
            %IF &_T=NUM %THEN %DO;
                %LET _F=%CMPRES(_&_I._);
                _VALUE=&_V||'="'||REPEAT('20'X,_N_-1)||COMPRESS(PUT(&_V,BEST.))||'"';
            %END;
            %ELSE %DO;
                %LET _F=%CMPRES($_&_I._);
                _VALUE='"'||TRIM(&_V)||'"="'||REPEAT('20'X,_N_-1)||TRIM(&_V)||'"';
            %END;
            IF _N_=1 THEN CALL EXECUTE( "PROC FORMAT; VALUE &_F" );
                          CALL EXECUTE( _VALUE );
            IF EOF   THEN CALL EXECUTE( ";RUN;" );
          RUN;
          %LET _F=%CMPRES(&_F..);
          PROC DATASETS LIBRARY=WORK NOLIST;
            MODIFY _DATASET_; FORMAT &_V &_F; DELETE _&_I;
          QUIT;
      %END;
    %END;

  %*****************************************************************************************************;
  %*   LIFEREG ANALYSIS                                                                                *;
  %*****************************************************************************************************;

  %* LIFEREG procedure produces the parameter estimates for the baseline hazard variables and          *;
  %* the covariates, and their estimated covariances (see matrix &CENS_OUTEST in Work library).        *;

  ODS EXCLUDE ALL;
  %IF %UPCASE(&PRINT)=YES %THEN %STR(ODS SELECT ALL;);
  PROC LIFEREG NAMELEN=200 COVOUT
       DATA = _DATASET_
       ORDER = FORMATTED
       OUTEST = &CENS._OUTEST(
         DROP = _LNLIKE_ &TIMEVARIABLE INTERCEPT _SCALE_
        WHERE = (UPCASE(_NAME_)^="INTERCEPT" AND UPCASE(_NAME_)^="SCALE")
       );
    CLASS &_BASELINE_MODELVARS_ &CLASSVARIABLES;
    MODEL &TIMEVARIABLE * &CENSORVARIABLE(&CENSORVALUE) =
          &BASELINE_MODEL
          &COVARIATE_MODEL
        / DIST=EXP NOINT MAXITER=500;
    ODS OUTPUT
      PARAMETERESTIMATES=&CENS._RR
      CONVERGENCESTATUS=&CENS._CONV;
  RUN;
  ODS SELECT ALL;

  %* RR *;

  DATA &CENS._RR;
    LENGTH Censor $ 32;
    SET &CENS._RR;
    CENSOR="&CENSORVARIABLE";
    PARAMETER=UPCASE(PARAMETER);
    IF  (INDEX(PARAMETER,"INTERCEPT")
      OR INDEX(PARAMETER,"SCALE")
      OR INDEX(PARAMETER,"WEIBULL")
     %IF &COHORTVARIABLE^=%STR() %THEN %STR( OR INDEX(PARAMETER,UPCASE("&COHORTVARIABLE")) );
      OR INDEX(PARAMETER,UPCASE("&PERIODVARIABLE"))) THEN DELETE;
    RR=EXP(-ESTIMATE);
    RR_LowerCL=EXP(-UPPERCL);
    RR_UpperCL=EXP(-LOWERCL);
  RUN;

  %*****************************************************************************************************;
  %*   SEPARATION OF PARAMETERS ESTIMATES AND THEIR COVARIANCES                                        *;
  %*****************************************************************************************************;

  %* The output matrices of the LIFEREG analysis (including both parameter estimates and their         *;
  %* estimated covariances) are divided into vector of parameter estimates (see vector &CENS_EST in    *;
  %* Work library) and covariance matrix of parameter estimates (see matrix &CENS_COVB in Work library)*;
  %* as these are needed separately in the calculation of PAF.                                         *;

  DATA  &CENS._EST
        &CENS._COVB;
    SET &CENS._OUTEST;
    IF      _TYPE_="PARMS" THEN OUTPUT &CENS._EST;
    ELSE IF _TYPE_="COV"   THEN OUTPUT &CENS._COVB;
  RUN;
  PROC TRANSPOSE DATA=&CENS._EST OUT=&CENS._EST;
  RUN;

  %*****************************************************************************************************;
  %*   FORMATION OF THEORETICAL DESIGN MATRIX                                                          *;
  %*****************************************************************************************************;

  %* A theoretical design matrix in which all individuals have the maximum amount of follow-up time    *;
  %* intervals is formed by duplicating the row related to the first follow-up time interval by the    *;
  %* number of follow-up time intervals. The only exception is the follow-up period variable which     *;
  %* identifies the follow-up time interval by indicating the cumulative follow-up time by the         *;
  %* beginning of each interval (see matrix _DESIGN_ in Work library). This matrix is used as an aid   *;
  %* in the formation of the original design matrices for baseline hazard variables and covariates.    *;

  %LET PERIODVALUES=0;
  %DO _I=1 %TO %EVAL(&PERIODCOUNT-1);
      %LET PERIODVALUES=&PERIODVALUES,%SYSEVALF(&_I*&PERIODLENGTH);
  %END;

  PROC SORT DATA=_DATASET_;
    BY &IDVARIABLE;
  RUN;
  DATA  _DESIGN_;
    SET _DATASET_;
    BY &IDVARIABLE;
    IF FIRST.&IDVARIABLE;
    DO &PERIODVARIABLE = &PERIODVALUES;
       OUTPUT;
    END;
  RUN;

  %*****************************************************************************************************;
  %*   FORMATION OF ORIGINAL DESIGN MATRIX FOR BASELINE HAZARD VARIABLES AND COVARIATES                *;
  %*****************************************************************************************************;

  %* A design matrix which indicates to which categories of the baseline hazard variables (follow-up   *;
  %* time intervals, birth cohorts, and their interactions) each individual belongs to on each         *;
  %* follow-up time interval and which values of the covariates each individual has are formed with    *;
  %* the help of the theoretical design matrix (see DESIGN_1 in in Work library).                      *;

  PROC LOGISTIC NAMELEN=200 OUTDESIGNONLY
       DATA=_DESIGN_
       OUTDESIGN=DESIGN_1(DROP=&CENSORVARIABLE);
    CLASS &_BASELINE_MODELVARS_ &CLASSVARIABLES / PARAM=GLM ORDER=FORMATTED;
    MODEL &CENSORVARIABLE =
          &BASELINE_MODEL
          &COVARIATE_MODEL
        / NOINT;
  RUN;

  %*****************************************************************************************************;
  %*   FORMATION OF MODIFIED DESIGN MATRIX FOR BASELINE HAZARD VARIABLES AND COVARIATES                *;
  %*****************************************************************************************************;

  %* A modified design matrix in which each individual has the reference value of the risk factor of   *;
  %* interest (given in the MODIFICATIONS) instead of their original value is formed (see              *;
  %* DESIGN_2 in Work library). This matrix is needed for the PAF analysis to represent the            *;
  %* situation after the hypothetical change of the risk factors of interest.                          *;

  DATA  _MODIFIED_;
    SET _DESIGN_;
    &MODIFICATIONS;
  RUN;

  PROC LOGISTIC NAMELEN=200 OUTDESIGNONLY
       DATA=_MODIFIED_
       OUTDESIGN=DESIGN_2(DROP=&CENSORVARIABLE);
    CLASS &_BASELINE_MODELVARS_ &CLASSVARIABLES / PARAM=GLM ORDER=FORMATTED;
    MODEL &CENSORVARIABLE =
          &BASELINE_MODEL
          &COVARIATE_MODEL
        / NOINT;
  RUN;

  %*****************************************************************************************************;
  %*   CORRESPONDENCE BETWEEN ORIGINAL AND MODIFIED DESIGN MATRIX                                      *;
  %*****************************************************************************************************;

  %* In the modified design matrix the columns related to the other values than the reference value    *;
  %* of the risk factors of interest are missing since none of the individuals belong to them.         *;
  %* Therefore, to make the original and modified design matrix to match, the modified design          *;
  %* matrix is completed to include also those zero columns (see DESIGN_2 in Work library).            *;

  DATA  DESIGN_2;
    SET DESIGN_1(OBS=0)
        DESIGN_2;
    ARRAY _COL_(*) _NUMERIC_;
    DO _I_=1 TO DIM(_COL_);
       IF MISSING(_COL_(_I_)) THEN _COL_(_I_)=0;
    END;
    DROP _I_;
  RUN;

  %****************************************************************************************************;
  %*   CORRESPONDENCE BETWEEN VECTOR OF PARAMETER ESTIMATES AND DESIGN MATRIX                         *;
  %****************************************************************************************************;

  %* Certain category related to the baseline hazard variables or the risk factors (most likely       *;
  %* (an interaction between the follow-up time interval and birth cohort) may be missing in the      *;
  %* original data. In that case this category exists in the design matrix, which inclues all         *;
  %* categories, but is missing from the vector of the parameter estimates. Therefore, to make the    *;
  %* vector of parameter estimates and the design matrix to match, the vector or parameter estimates  *;
  %* is completed to include also those zero cells (see _PARMS_ in Work library).                     *;

  %* FULL PARAMETER VECTOR FROM DESIGN MATRIX *;

  %* COLUMNS *;
  DATA _PARMS_;
    SET DESIGN_1(OBS=0);
  RUN;

  %* ROWS *;
  PROC TRANSPOSE DATA=_PARMS_ OUT=_PARMS_T;
  RUN;
  DATA _PARMS_T;
    SET _PARMS_T;
    __PARM__ORDER__=PUT(_N_,Z3.);
  RUN;

  PROC SQL;
   %* CREATE TABLE OF ALL BETA ESTIMATES, RETAIN ORIGINAL ORDER *;
    CREATE TABLE _E_ AS
      SELECT A._NAME_ AS _A_NAME_, B.*
        FROM _PARMS_T A
        LEFT JOIN &CENS._EST B
          ON (A._NAME_=B._NAME_)
       ORDER BY A.__PARM__ORDER__;
    %* CREATE TABLE OF ALL COVARIANCE ESTIMATES, RETAIN ORIGINAL ORDER *;
    CREATE TABLE _C_ AS
      SELECT A._NAME_ AS _A_NAME_, B.*
        FROM _PARMS_T A
        LEFT JOIN &CENS._COVB B
          ON (A._NAME_=B._NAME_)
       ORDER BY A.__PARM__ORDER__;
  QUIT;

  %* CONVERT MISSING ROWS AND COLUMNS TO ZEROS *;

  DATA &CENS._EST;
    LENGTH _A_NAME_ $200;
    SET _E_;
      IF MISSING(&TIMEVARIABLE) THEN &TIMEVARIABLE=0;
    KEEP _A_NAME_ _NUMERIC_;
  RUN;

  DATA &CENS._COVB;
    LENGTH _A_NAME_ $200;
    SET _PARMS_  _C_;
      ARRAY __COL__(*) _NUMERIC_;
      DO __I__=1 TO DIM(__COL__);
         IF MISSING( __COL__(__I__) ) THEN __COL__(__I__)=0;
      END;
    DROP __I__;
    KEEP _A_NAME_ _NUMERIC_;
  RUN;

  ODS RESULTS;
  OPTIONS SOURCE;
%MEND EST_MATRIX;
%*************************************************************************************************************;
