%*************************************************************************************************************;
%*                                                                                                           *;
%* Article: SAS macros for calculation of Population Attributable Fraction (PAF) in a cohort study design    *;
%* Authors: Maarit A. Laaksonen, Esa Virtala, Paul Knekt, Hannu Oja, and Tommi Hrknen                      *;
%*                                                                                                           *;
%*                                                                                                           *;
%* EST_PAF_D                                                                                                 *;
%*                                                                                                           *;
%* Population Attributable Fraction (PAF) for disease incidence with piecewise constant hazards model.       *;
%*                                                                                                           *;
%*                                                                                                           *;
%* Notation:                                                                                                 *;
%*   n = number of individuals, i=1,...,n                                                                    *;
%*   J = number of follow-up intervals, j=1,...,J                                                            *;
%*   C = number of birth cohorts, c=1,...,C                                                                  *;
%*   m = number of (all categories of) covariates                                                            *;
%*                                                                                                           *;
%* INPUT:                                                                                                    *;
%*                                                                                                           *;
%*   GROUP        = Group identification number                                                              *;
%*                  (needed for analysis of potential effect modification, see EST_PAF_DIFF_D)               *;
%*                                                                                                           *;
%*   DES          = Design matrix for baseline hazard parameters and observed covariates                     *;
%*                  (n*J)*(C+J+C*J+m) -matrix                                                                *;
%*   DES_STAR     = Design matrix for baseline hazard parameters and modified covariates                     *;
%*                  (n*J)*(C+J+C*J+m) -matrix                                                                *;
%*   EST_M        = Baseline hazard and covariate estimates related to mortality                             *;
%*                  (C+J+C*J+m)*1 -vector                                                                    *;
%*   EST_D        = Baseline hazard and covariate estimates related to disease                               *;
%*                  (C+J+C*J+m)*1 -vector                                                                    *;
%*   COVB_M       = Covariance matrix for baseline hazard and covariate estimates related to mortality       *;
%*                  (C+J+C*J+m)*(C+J+C*J+m) -matrix                                                          *;
%*   COVB_D       = Covariance matrix for baseline hazard and covariate estimates related to disease         *;
%*                  (C+J+C*J+m)*(C+J+C*J+m) -matrix                                                          *;
%*                                                                                                           *;
%*   PERIODCOUNT  = Number of follow-up intervals                                                            *;
%*                                                                                                           *;
%*   PERIODLENGTH = Length of follow-up intervals                                                            *;
%*                                                                                                           *;
%*   DELTALENGTH  = Length of follow-up                                                                      *;
%*                                                                                                           *;
%* OUTPUT:                                                                                                   *;
%*                                                                                                           *;
%*   RESULT       = Name of vector where results from PAF analysis are gathered                              *;
%*                                                                                                           *;
%*   SAS datasets needed for EST_PAF_DIFF_D:                                                                 *;
%*                                                                                                           *;
%*   PAF_&GROUP                                                                                              *;
%*                                                                                                           *;
%*   DPAF_&GROUP_GAMMA_M                                                                                     *;
%*                                                                                                           *;
%*   DPAF_&GROUP_GAMMA_D                                                                                     *;
%*                                                                                                           *;
%*************************************************************************************************************;

%MACRO EST_PAF_D(
  GROUP        = ,
  DES          = ,
  DES_STAR     = ,
  EST_M        = ,
  EST_D        = ,
  COVB_M       = ,
  COVB_D       = ,
  PERIODCOUNT  = ,
  PERIODLENGTH = ,
  DELTALENGTH  = ,
  RESULT       =
);

%* Adjust memory *;
%LET _MEM_=%SCAN( &DES, 1, %STR(%() );
PROC SQL NOPRINT;
  SELECT PUT(2*FILESIZE,32.) INTO :_FILESIZE_ FROM SASHELP.VTABLE WHERE LIBNAME="WORK" AND MEMNAME="&_MEM_";
QUIT;

%*****************************************************************************************;

PROC IML WORKSIZE=&_FILESIZE_;

  %*******************************************************;
  %**************  Define utility modules  ***************;
  %*******************************************************;

  start _CUMSUM_( inmatrix, outmatrix, groupsize );

     %* Cumulative column sums related to each invidual               *;
     %* GROUPSIZE = Number of columns related to the same individual, *;
     %*             if missing, assumed to include all rows.          *;

     if missing(groupsize) then groupsize = nrow( inmatrix );

     ncolumns  = ncol( inmatrix );
     outmatrix = t( btran( inmatrix, groupsize, ncolumns ) );
     ncolout   = ncol( outmatrix );
     do _i=2 to ncolout;
        outmatrix[ ,_i] = outmatrix[ ,_i-1] + outmatrix[ ,_i];
     end;
     outmatrix = btran( t(outmatrix), groupsize, ncolumns );

  finish _CUMSUM_;

  %*******************************************************;

  start _LAG_( inmatrix, outmatrix, groupsize, lag, value );

     %* Preceding column value by individual                                *;
     %* GROUPSIZE = Number of columns related to the same individual,       *;
     %*             if missing, assumed to include all rows.                *;
     %* LAG       = length of lag in choosing the preceding column value,   *;
     %*             if missing, assumed to be 1                             *;
     %* VALUE     = value replacing the possible missing value at beginning *;

     if missing(groupsize) then groupsize = nrow( inmatrix );
     if missing(lag)       then lag = 1;

     if groupsize<=1 then
        outmatrix = j( nrow(inmatrix), ncol(inmatrix), value );
     else do;
        ncolumns  = ncol( inmatrix );
        outmatrix = t( btran( inmatrix, groupsize, ncolumns ) );
        nrowout   = nrow( outmatrix  );
        outmatrix = btran( t( j(nrowout, lag, value) || outmatrix[ ,1:groupsize-lag] ), groupsize, ncolumns );
     end;

  finish _LAG_;

  %***************************************************************;
  %* Define module for calculation of PAF for disease incidence  *;
  %***************************************************************;

  start _PAF_;

     %* POINT ESTIMATE OF PAF *;

     %* Hazard of death for observed and modified ('star') covariate values *;
     lambda_M      = exp (DES * (-EST_M));
     lambda_M_star = exp (DES_STAR * (-EST_M));

     %* Hazard of disease incidence for observed and modified ('star') covariate values *;
     lambda_D= exp (DES * (-EST_D));
     lambda_D_star = exp (DES_STAR * (-EST_D));

     %* Sum of hazard of death and hazard of disease incidence *;
     lambda_M_plus_D      = lambda_M      + lambda_D;
     lambda_M_plus_D_star = lambda_M_star + lambda_D_star;

     %* Probability that the observed event is disease *;
     dis_prob      = lambda_D      / lambda_M_plus_D;
     dis_prob_star = lambda_D_star / lambda_M_plus_D_star;

     %* Exposure at different follow-up time intervals until the end of follow-up *;
     lambda_M_plus_D_delta      = lambda_M_plus_D      # DELTA;
     lambda_M_plus_D_star_delta = lambda_M_plus_D_star # DELTA;

     %* Individual total exposure by the end of each follow-up time-interval: *;
     %* sum of exposure at previous follow-up time intervals *;
     run _CUMSUM_( lambda_M_plus_D_delta, cs_lambda_M_plus_D_delta, &PERIODCOUNT );
     run _CUMSUM_( lambda_M_plus_D_star_delta, cs_lambda_M_plus_D_star_delta, &PERIODCOUNT );

     %* Survival until time a_j: S_j ja S_j* *;
     S_t2      = exp (-cs_lambda_M_plus_D_delta);
     S_t2_star = exp (-cs_lambda_M_plus_D_star_delta);

     %* Survival until time a_j-1: S_j-1 ja S_j-1* (calculated with the help of S_j ja S_j*) *;
     run _LAG_(  S_t2,  S_t1, &PERIODCOUNT, 1, 1 );
     run _LAG_( S_t2_star, S_t1_star, &PERIODCOUNT, 1, 1 );

     %* Point estimate PAF *;
     I = ( dis_prob # (S_t1 - S_t2) )[:,];
     I_star = ( dis_prob_star # (S_t1_star - S_t2_star) )[:,];
     PAF = 1 - (I_star/I);


     %* CONFIDENCE INTERVAL OF PAF *;

     %* Derivative of I w.r.t. gamma_M *;

     %* preparation of parameters related to mortality needed for the derivation *;
     lambda_M_delta_t2_Z = (lambda_M # DELTA) # DES;
     run _CUMSUM_( lambda_M_delta_t2_Z, cs_lambda_M_delta_t2_Z, &PERIODCOUNT );
     lambda_M_star_delta_t2_Z_star = (lambda_M_star # DELTA) # DES_STAR;
     run _CUMSUM_( lambda_M_star_delta_t2_Z_star, cs_lambda_M_star_delta_t2_Z_star, &PERIODCOUNT );
     run _LAG_( cs_lambda_M_delta_t2_Z, cs_lambda_M_delta_t1_Z, &PERIODCOUNT, 1, 0 );
     run _LAG_( cs_lambda_M_star_delta_t2_Z_star, cs_lambda_M_star_delta_t1_Z_star, &PERIODCOUNT, 1, 0 );

     dI_gamma_M =
        ( -DES # lambda_D # lambda_M / (lambda_M_plus_D##2) # (S_t1 - S_t2)
          + dis_prob # ( S_t2 # cs_lambda_M_delta_t2_Z
                       - S_t1 # cs_lambda_M_delta_t1_Z ) ) [:,];

     dI_star_gamma_M =
        ( -DES_STAR # lambda_D_star # lambda_M_star / (lambda_M_plus_D_star##2) # (S_t1_star - S_t2_star)
          + dis_prob_star # ( S_t2_star # cs_lambda_M_star_delta_t2_Z_star
                            - S_t1_star # cs_lambda_M_star_delta_t1_Z_star ) ) [:,];

     %* Derivative of I w.r.t. gamma_D *;

     %* preparation of parameters related to disease needed for the derivation *;
     lambda_D_delta_t2_Z = (lambda_D # DELTA) # DES;
     run _CUMSUM_( lambda_D_delta_t2_Z, cs_lambda_D_delta_t2_Z, &PERIODCOUNT );
     lambda_D_star_delta_t2_Z_star = (lambda_D_star # DELTA) # DES_STAR;
     run _CUMSUM_( lambda_D_star_delta_t2_Z_star, cs_lambda_D_star_delta_t2_Z_star, &PERIODCOUNT );
     run _LAG_( cs_lambda_D_delta_t2_Z, cs_lambda_D_delta_t1_Z, &PERIODCOUNT, 1, 0 );
     run _LAG_( cs_lambda_D_star_delta_t2_Z_star, cs_lambda_D_star_delta_t1_Z_star, &PERIODCOUNT, 1, 0 );

     dI_gamma_D =
        ( DES # lambda_D # lambda_M / (lambda_M_plus_D##2) # (S_t1 - S_t2)
          + dis_prob # ( S_t2 # cs_lambda_D_delta_t2_Z
                       - S_t1 # cs_lambda_D_delta_t1_Z ) ) [:,];

     dI_star_gamma_D =
        ( DES_STAR # lambda_D_star # lambda_M_star / (lambda_M_plus_D_star##2) # (S_t1_star - S_t2_star)
          + dis_prob_star # ( S_t2_star # cs_lambda_D_star_delta_t2_Z_star
                             - S_t1_star # cs_lambda_D_star_delta_t1_Z_star ) ) [:,];

     %* Derivate of PAF w.r.t. gamma_M *;
     dPAF_gamma_M  = (dI_gamma_M # I_star - dI_star_gamma_M # I) / I##2;

     %* Derivate of PAF w.r.t. gamma_D *;
     dPAF_gamma_D  = (dI_gamma_D # I_star - dI_star_gamma_D # I) / I##2;

     %* Variance and standard error of PAF *;
     var_PAF = dPAF_gamma_D * COVB_D * t(dPAF_gamma_D) + dPAF_gamma_M * COVB_M * t(dPAF_gamma_M);
     se_PAF  = sqrt(var_PAF);

     %* Confidence limits for PAF *;
     PAF_CL_l = PAF - probit(0.975) * se_PAF;
     PAF_CL_u = PAF + probit(0.975) * se_PAF;


     %* CONFIDENCE INTERVAL OF PAF USING COMPLEMENTARY LOGARITHMIC TRANSFORMATION *;

     %* Logarithmic transformation of PAF: log(1-PAF) *;
     lPAF = log(1-PAF);

     %* Derivate of log(1-PAF) w.r.t gamma_M *;
     dlPAF_gamma_M = dI_star_gamma_M / I_star - dI_gamma_M / I;

     %* Derivate of log(1-PAF) w.r.t gamma_D *;
     dlPAF_gamma_D = dI_star_gamma_D / I_star - dI_gamma_D / I;

     %* Variance and standard error of log(1-PAF) *;
     var_lPAF = dlPAF_gamma_D * COVB_D * t(dlPAF_gamma_D) + dlPAF_gamma_M * COVB_M * t(dlPAF_gamma_M);
     se_lPAF  = sqrt(var_lPAF);

     %* Confidence limits for inverse log(1-PAF) *;
     lPAF_CL_l = 1 - exp(lPAF + probit(0.975) * se_lPAF);
     lPAF_CL_u = 1 - exp(lPAF - probit(0.975) * se_lPAF);

  finish _PAF_;

  %*******************************************************;
  %* Define main module                                  *;
  %*******************************************************;

  start _MAIN_;
    FREE / ;

    %*******************************************************;
    %**************  Read input matrices  ******************;
    %*******************************************************;

    %* Design matrix for baseline hazard parameters and observed covariates *;
    use &DES;
      read  all into DES;
    close;

    %* Design matrix for baseline hazard parameters and modified covariates *;
    use &DES_STAR;
      read  all into DES_STAR;
    close;

    %* Baseline hazard and covariate estimate vector related to mortality *;
    use &EST_M;
      read  all into EST_M;
    close;

    %* Baseline hazard and covariate estimate vector related to disease *;
    use &EST_D;
      read  all into EST_D;
    close;

    %* Covariance matrix for baseline hazard and covariate estimates related to mortality *;
    use &COVB_M;
      read  all into COVB_M;
    close;

    %* Covariance matrix for baseline hazard and covariate estimates related to disease *;
    use &COVB_D;
      read  all into COVB_D;
    close;

    %* Formation of delta vector: exposure at different follow-up time intervals until the end of follow-up *;
    %IF &PERIODCOUNT=1 %THEN %DO;
       DELTA  = j(1, 1, 0);
       DELTA[1, 1] = &DELTALENGTH;
    %END;
    %ELSE %DO;
       DELTA  = j(&PERIODCOUNT, 1, 0);
       do _period_ = 1 to &PERIODCOUNT;
          DELTA[_period_, 1] = &PERIODLENGTH;
          _time_ = _period_ * DELTA[_period_, 1];
          if _time_ > &DELTALENGTH then do;
             DELTA[_period_, 1] = &DELTALENGTH - _time_ + &PERIODLENGTH;
             _time_   = &DELTALENGTH;
             _period_ = &PERIODCOUNT+1;
          end;
      end;
    %END;

    %* Number of individuals in the data *;
    _NOBS_ = NROW(DES) / &PERIODCOUNT;

    %* Duplication of delta vector to match design matrices *;
    DELTA = repeat(DELTA, _NOBS_);


   %*******************************************************;
    %****** Calculate PAF and its confidence limits  *******;
    %*******************************************************;

    run _PAF_;


    %*******************************************************;
    %*** Gather results of the PAF estimation to dataset ***;
    %*******************************************************;

    _PAF = PAF || se_lPAF || lPAF_cl_l || lPAF_cl_u;

    create &RESULT from _PAF [colname={"PAF" "SE_lPAF" "lPAF_LowerCL" "lPAF_UpperCL"}];
      append from _PAF;
    close;


    %*******************************************************;
    %****** Gather results needed for EST_PAF_DIFF_D  ******;
    %*******************************************************;

    %IF &GROUP^=%STR( ) %THEN %DO;
        create PAF_&GROUP from PAF;
          append from PAF;
        close;
        create dPAF_&GROUP._gamma_D from dPAF_gamma_D;
          append from dPAF_gamma_D;
        close;
        create dPAF_&GROUP._gamma_M from dPAF_gamma_M;
          append from dPAF_gamma_M;
        close;
    %END;

    %*******************************************************;

    FREE / ;
  finish _MAIN_;

  %*******************************************************;
  %* Run main module                                     *;
  %*******************************************************;

  run _MAIN_;

  %*******************************************************;

QUIT;

%*****************************************************************************************;

DATA &RESULT;
  LENGTH Period Type $ 32;
  SET &RESULT;
  PERIOD = "0-<&DELTALENGTH";
  TYPE = "Cumulative (delta)";
  DROP TYPE;
RUN;

%MEND EST_PAF_D;
%*************************************************************************************************************;
