%*************************************************************************************************************;
%*                                                                                                           *;
%* Article: SAS macros for calculation of Population Attributable Fraction (PAF) in a cohort study design    *;
%* Authors: Maarit A. Laaksonen, Esa Virtala, Paul Knekt, Hannu Oja, and Tommi Hrknen                      *;
%*                                                                                                           *;
%*                                                                                                           *;
%* EST_PAF_DIFF_M                                                                                            *;
%*                                                                                                           *;
%* Difference of PAF estimates for total mortality between categories of certain factor and its              *;
%* statistical significance.                                                                                 *;
%*                                                                                                           *;
%*                                                                                                           *;
%* INPUT:                                                                                                    *;
%*                                                                                                           *;
%*   PAF_G1          = PAF estimate for group 1                                                              *;
%*                                                                                                           *;
%*   PAF_G2          = PAF estimate for group 2                                                              *;
%*                                                                                                           *;
%*   DPAF_G1_GAMMA   = Derivative of PAF for group 1 w.r.t. parameters related to mortality                  *;
%*                                                                                                           *;
%*   DPAF_G2_GAMMA   = Derivative of PAF for group 2 w.r.t. parameters related to mortality                  *;
%*                                                                                                           *;
%*   COVB            = Covariance matrix for covariate and baseline hazard estimates for mortality           *;
%*                                                                                                           *;
%*   DELTALENGTH     = Length of follow-up                                                                   *;
%*                                                                                                           *;
%* OUTPUT:                                                                                                   *;
%*                                                                                                           *;
%*   RESULT          = Name of vector where results from PAF analysis are gathered                           *;
%*                                                                                                           *;
%*************************************************************************************************************;

%MACRO EST_PAF_DIFF_M(
  PAF_G1        = ,
  PAF_G2        = ,
  DPAF_G1_GAMMA = ,
  DPAF_G2_GAMMA = ,
  COVB          = ,
  DELTALENGTH   = ,
  RESULT        =
);

%***********************************************************************************************;

PROC IML;
  FREE / ;

  %*******************************************************;
  %**************  Read input matrices  ******************;
  %*******************************************************;

  %* PAF for group 1 *;
  USE &PAF_G1;
    READ ALL INTO PAF_G1;
  CLOSE;

  %* PAF for group 2 *;
  USE &PAF_G2;
    READ ALL INTO PAF_G2;
  CLOSE;

  %* Derivative of PAF for group 1 w.r.t. parameters related to mortality *;
  USE &dPAF_G1_GAMMA;
    READ ALL INTO dPAF_G1_GAMMA;
  CLOSE;

  %* Derivative of PAF for group 2 w.r.t. parameters related to mortality *;
  USE &dPAF_G2_GAMMA;
    READ ALL INTO dPAF_G2_GAMMA;
  CLOSE;

  %* Covariance matrix for covariate and baseline hazard estimates related to mortality *;
  USE &COVB;
    READ ALL INTO COVB;
  CLOSE;


  %***************************************************************;
  %* Calculate PAF difference, its confidence limits and p-value *;
  %***************************************************************;

  %* Point estimate of PAF difference *;
  PAF_diff = PAF_G1 - PAF_G2;

  %* Variance and standard error of PAF difference *;
  var_PAF_diff = t(dPAF_G2_gamma - dPAF_G1_gamma) * COVB * (dPAF_G2_gamma - dPAF_G1_gamma);
  se_PAF_diff  = sqrt(var_PAF_diff);

  %* Confidence limits for PAF difference *;
  PAF_diff_CL_l = PAF_diff - probit(0.975) * se_PAF_diff;
  PAF_diff_CL_u = PAF_diff + probit(0.975) * se_PAF_diff;

  %* p-value for PAF difference *;
  PAF_diff_Z = abs( PAF_diff / se_PAF_diff );
  PAF_diff_PVALUE = 2 * (1 - probnorm(PAF_diff_Z));


  %*******************************************************;
  %*** Gather results of the PAF estimation to dataset ***;
  %*******************************************************;

  _PAF_diff = PAF_g1 || PAF_g2 || PAF_diff || se_PAF_diff || PAF_diff_cl_l || PAF_diff_cl_u || PAF_diff_z || PAF_diff_pvalue;

  create &RESULT from _PAF_diff [colname={"PAF_Group_A" "PAF_Group_B" "PAF_Diff" "SE_PAF_Diff" "PAF_Diff_LowerCL" "PAF_Diff_UpperCL" "PAF_Diff_Z" "PAF_Diff_Pvalue"}];
    append from _PAF_diff;
  close;

  %*******************************************************;

  FREE / ;
QUIT;

%***********************************************************************************************;

DATA &RESULT;
  LENGTH Period $ 32;
  SET &RESULT;
  PERIOD = "0-<&DELTALENGTH";
 RUN;

%MEND EST_PAF_DIFF_M;
%*************************************************************************************************************;
