%*************************************************************************************************************;
%*                                                                                                           *;
%* Article: SAS macros for calculation of Population Attributable Fraction (PAF) in a cohort study design    *;
%* Authors: Maarit A. Laaksonen, Esa Virtala, Paul Knekt, Hannu Oja, and Tommi Hrknen                      *;
%*                                                                                                           *;
%*                                                                                                           *;
%*  EST_PAF_M                                                                                                *;
%*                                                                                                           *;
%* Population Attributable Fraction (PAF) for total mortality with piecewise constant hazards model.         *;
%*                                                                                                           *;
%*                                                                                                           *;
%* Notation:                                                                                                 *;
%*   n = number of individuals, i=1,...,n                                                                    *;
%*   J = number of follow-up intervals, j=1,...,J                                                            *;
%*   C = number of birth cohorts, c=1,...,C                                                                  *;
%*   m = number of (all categories of) covariates                                                            *;
%*                                                                                                           *;
%* INPUT:                                                                                                    *;
%*                                                                                                           *;
%*   GROUP        = Group identification number                                                              *;
%*                  (needed for analysis of potential effect modification, see EST_PAF_DIFF_M)               *;
%*                                                                                                           *;
%*   DES          = Design matrix for baseline hazard parameters and observed covariates                     *;
%*                             (n*J)*(C+J+C*J+m) -matrix                                                     *;
%*   DES_STAR     = Design matrix for baseline hazard parameters and modified covariates                     *;
%*                             (n*J)*(C+J+C*J+m) -matrix                                                     *;
%*   EST          = Baseline hazard and covariate estimates related to mortality                             *;
%*                             (C+J+C*J+m)*1 -vector                                                         *;
%*   COVB         = Covariance matrix for baseline hazard and covariate estimates related to mortality       *;
%*                             (C+J+C*J+m)*(C+J+C*J+m) -matrix                                               *;
%*                                                                                                           *;
%*   PERIODCOUNT  = Number of follow-up intervals                                                            *;
%*                                                                                                           *;
%*   PERIODLENGTH = Length of follow-up intervals                                                            *;
%*                                                                                                           *;
%*   DELTALENGTH  = Length of follow-up                                                                      *;
%*                                                                                                           *;
%* OUTPUT:                                                                                                   *;
%*                                                                                                           *;
%*   RESULT       = Name of vector where results from PAF analysis are gathered                              *;
%*                                                                                                           *;
%*   SAS datasets needed for EST_PAF_DIFF_M:                                                                 *;
%*                                                                                                           *;
%*   PAF_&GROUP                                                                                              *;
%*                                                                                                           *;
%*   DPAF_&GROUP_GAMMA                                                                                       *;
%*                                                                                                           *;
%*************************************************************************************************************;

%MACRO EST_PAF_M(
  GROUP        = ,
  DES          = ,
  DES_STAR     = ,
  EST          = ,
  COVB         = ,
  PERIODCOUNT  = ,
  PERIODLENGTH = ,
  DELTALENGTH  = ,
  RESULT       =
);

%* Adjust memory *;
%LET _MEM_=%UPCASE(%SCAN( &DES, 1, %STR(%() ));
PROC SQL NOPRINT;
  SELECT PUT(2*FILESIZE,32.) INTO :_FILESIZE_ FROM SASHELP.VTABLE WHERE LIBNAME="WORK" AND MEMNAME="&_MEM_";
QUIT;

%*******************************************************;

PROC IML WORKSIZE=&_FILESIZE_;

  %*******************************************************;
  %**************  Define utility modules  ***************;
  %*******************************************************;

  start _COLSUM_( inmatrix, outmatrix, groupsize );

    %* Column sums related to each invidual                          *;
    %* GROUPSIZE = Number of columns related to the same individual, *;
    %*             if missing, assumed to include all rows.          *;

    if missing(groupsize) then groupsize = nrow( inmatrix );

    ncolumns  = ncol( inmatrix );
    outmatrix = btran( btran( inmatrix, groupsize, ncolumns)[+, ], 1, ncolumns );

  finish _COLSUM_;

  %*******************************************************;

  start _PAF_RESULT_;

    %* Results of the PAF estimation to dataset *;

    Result[1,1] = PAF;
    Result[1,2] = se_lPAF;
    Result[1,3] = lPAF_CL_l;
    Result[1,4] = lPAF_CL_u;
    Result[1,5] = _PAF_Type;
    Result[1,6] = _TIME_1;
    Result[1,7] = _TIME_2;

    setout &RESULT;
    append from Result;

  finish _PAF_RESULT_;

  %*******************************************************;
  %* Define module for calculation of PAF for mortality  *;
  %*******************************************************;

  start _PAF_;

    %* POINT ESTIMATE OF PAF *;

    %* Hazard of death for observed and modified ('star') covariate values *;
    lambda      = exp (DES * (-EST));
    lambda_star = exp (DES_STAR * (-EST));

    %* Exposure at different follow-up time intervals until time t *;
    lambda_delta_t1      = lambda # DELTA_1;
    lambda_star_delta_t1 = lambda_star # DELTA_1;

    %* Exposure at different follow-up time intervals until time t+delta_t *;
    lambda_delta_t2      = lambda # DELTA_2;
    lambda_star_delta_t2 = lambda_star # DELTA_2;

    %* Individual total exposure until time t:               *;
    %* sum of exposure at different follow-up time intervals *;
    run _COLSUM_( lambda_delta_t1, sum_lambda_delta_t1, &PERIODCOUNT );
    run _COLSUM_( lambda_star_delta_t1, sum_lambda_star_delta_t1, &PERIODCOUNT );

    %* Individual total exposure until time t+delta_t:       *;
    %* sum of exposure at different follow-up time intervals *;
    run _COLSUM_( lambda_delta_t2, sum_lambda_delta_t2, &PERIODCOUNT );
    run _COLSUM_( lambda_star_delta_t2, sum_lambda_star_delta_t2, &PERIODCOUNT );

    %* Survival until time t *;
    S_t1      = exp (-sum_lambda_delta_t1);
    S_t1_star = exp (-sum_lambda_star_delta_t1);

    %* Survival until time t+delta_t *;
    S_t2      = exp (-sum_lambda_delta_t2);
    S_t2_star = exp (-sum_lambda_star_delta_t2);

    %* Point estimate PAF *;
    I = (S_t1 - S_t2) [:,];
    I_star = (S_t1_star - S_t2_star) [:,];
    PAF = 1 - (I_star/I);


    %* CONFIDENCE INTERVAL OF PAF *;

    %* Derivative of I w.r.t. gamma *;

    %* preparation of parameters needed for the derivation *;
    lambda_delta_t1_Z = lambda_delta_t1 # DES;
    run _COLSUM_( lambda_delta_t1_Z, sum_lambda_delta_t1_Z, &PERIODCOUNT );
    lambda_star_delta_t1_Z_star = lambda_star_delta_t1 # DES_STAR;
    run _COLSUM_( lambda_star_delta_t1_Z_star, sum_lambda_star_delta_t1_Z_star, &PERIODCOUNT );
    lambda_delta_t2_Z = lambda_delta_t2 # DES;
    run _COLSUM_( lambda_delta_t2_Z, sum_lambda_delta_t2_Z, &PERIODCOUNT );
    lambda_star_delta_t2_Z_star = lambda_star_delta_t2 # DES_STAR;
    run _COLSUM_( lambda_star_delta_t2_Z_star, sum_lambda_star_delta_t2_Z_star, &PERIODCOUNT );

    dI_gamma = t( (S_t2 # sum_lambda_delta_t2_Z
               - S_t1 # sum_lambda_delta_t1_Z) [:,] );
    dI_star_gamma = t( (S_t2_star # sum_lambda_star_delta_t2_Z_star
                    - S_t1_star # sum_lambda_star_delta_t1_Z_star) [:,] );

    %* Derivative of PAF w.r.t gamma *;
    dPAF_gamma = (dI_gamma # I_star - dI_star_gamma # I) / I##2;

    %* Variance and standard error of PAF *;
    var_PAF = t(dPAF_gamma) * COVB * dPAF_gamma;
    se_PAF  = sqrt(var_PAF);

    %* Confidence limits for PAF *;
    PAF_CL_l = PAF - probit(0.975) * se_PAF;
    PAF_CL_u = PAF + probit(0.975) * se_PAF;


    %* CONFIDENCE INTERVAL OF PAF USING COMPLEMENTARY LOGARITHMIC TRANSFORMATION *;

    %* Logarithmic transformation of PAF: log(1-PAF) *;
    lPAF = log(1-PAF);

    %* Derivative of log(1-PAF) w.r.t gamma *;

    dlPAF_gamma = dI_star_gamma / I_star - dI_gamma / I;

    %* Variance and standard error of log(1-PAF) *;
    var_lPAF = t(dlPAF_gamma) * COVB * dlPAF_gamma;
    se_lPAF  = sqrt(var_lPAF);

    %* Confidence limits for inverse log(1-PAF) *;
    lPAF_CL_l = 1 - exp(lPAF + probit(0.975) * se_lPAF);
    lPAF_CL_u = 1 - exp(lPAF - probit(0.975) * se_lPAF);

  finish _PAF_;

  %*******************************************************;
  %* Define main module                                  *;
  %*******************************************************;

  start _MAIN_;
    FREE / ;

    %*******************************************************;
    %**************  Read input matrices  ******************;
    %*******************************************************;

    %* Design matrix for baseline hazard parameters and observed covariates *;
    use &DES;
      read  all into DES;
    close;

    %* Design matrix for baseline hazard parameters and modified covariates *;
    use &DES_STAR;
      read  all into DES_STAR;
    close;

    %* Baseline hazard and covariate estimate vector *;
    use &EST;
      read  all into EST;
    close;

    %* Covariance matrix for baseline hazard and covariate estimates *;
    use &COVB;
      read  all into COVB;
    close;

    %* If only one period *;
    %IF &PERIODCOUNT=1 %THEN %DO;
        %LET PERIODLENGTH=&DELTALENGTH;
        %IF &PERIODLENGTH=%STR() %THEN %DO;
            %PUT %STR( );
            %PUT ERROR: PERIODCOUNT=&PERIODCOUNT..;
            %PUT ERROR- PERIODLENGTH=&PERIODLENGTH..;
            %PUT ERROR- DELTALENGTH=&DELTALENGTH..;
            %PUT %STR( );
            %GOTO _EXIT_;
        %END;
    %END;

    %*******************************************************;
    %**************  Prepare output data set  **************;
    %*******************************************************;

    Result = J(1,7,.);
    create &RESULT
      from Result [colname={"PAF" "SE_lPAF" "lPAF_LowerCL" "lPAF_UpperCL" "PAF_Type" "Time_1" "Time_2"}];


    %*******************************************************;
    %****** Calculate PAF and its confidence limits  *******;
    %*******************************************************;

    %* Number of individuals in the data *;
    _NOBS_ = NROW(DES) / &PERIODCOUNT;

    %* Piecewise *;

    _DELTA_1 = J(&PERIODCOUNT, 1, 0);
    _DELTA_2 = J(&PERIODCOUNT, 1, 0);

    do _PERIOD=1 to &PERIODCOUNT;

       _DELTA_2[_PERIOD, 1] = &PERIODLENGTH;
       _TIME_2 = _PERIOD*_DELTA_2[_PERIOD, 1];
       _TIME_1 = _TIME_2-&PERIODLENGTH;

       DELTA_1 = repeat(_DELTA_1, _NOBS_);
       DELTA_2 = repeat(_DELTA_2, _NOBS_);

       _PAF_Type=1;

       run _PAF_;
       run _PAF_RESULT_;

       _DELTA_1 = _DELTA_2;

    end;

    %* Cumulative *;

    _DELTA_1 = J(&PERIODCOUNT, 1, 0);
    _DELTA_2 = J(&PERIODCOUNT, 1, 0);

    do _PERIOD=1 to &PERIODCOUNT;

       _DELTA_2[_PERIOD, 1] = &PERIODLENGTH;
       _TIME_2 = _PERIOD*_DELTA_2[_PERIOD, 1];
       _TIME_1 = 0;

       DELTA_1 = repeat(_DELTA_1, _NOBS_);
       DELTA_2 = repeat(_DELTA_2, _NOBS_);

       _PAF_Type=2;

       run _PAF_;
       run _PAF_RESULT_;

    end;

    %* Cumulative (using DELTALENGTH) *;

    %IF &DELTALENGTH^=%STR( ) %THEN %DO;

        _DELTA_1 = J(&PERIODCOUNT, 1, 0);
        _DELTA_2 = J(&PERIODCOUNT, 1, 0);

        do _PERIOD=1 to &PERIODCOUNT;

           _DELTA_2[_PERIOD, 1] = &PERIODLENGTH;
           _TIME_2 = _PERIOD*_DELTA_2[_PERIOD, 1];
           _TIME_1 = 0;

           if _TIME_2>&DELTALENGTH then do;
              _DELTA_2[_PERIOD, 1] = &DELTALENGTH-_TIME_2+&PERIODLENGTH;
              _TIME_2 = &DELTALENGTH;
              _PERIOD = &PERIODCOUNT+1;
           end;

        end;

        DELTA_1 = repeat(_DELTA_1, _NOBS_);
        DELTA_2 = repeat(_DELTA_2, _NOBS_);

        _PAF_Type=3;

        run _PAF_;
        run _PAF_RESULT_;

    %END;

    %*******************************************************;
    %****** Gather results needed for EST_PAF_DIFF_M  ******;
    %*******************************************************;

    %IF &GROUP^=%STR( ) %THEN %DO;
        create PAF_&GROUP from PAF;
          append from PAF;
        close;
        create dPAF_&GROUP._gamma from dPAF_gamma;
          append from dPAF_gamma;
        close;
    %END;

    %*******************************************************;

    FREE / ;
  finish _MAIN_;

  %*******************************************************;
  %* Run main module                                     *;
  %*******************************************************;

  run _MAIN_;

  %*******************************************************;

QUIT;

%*******************************************************;

DATA &RESULT;
   LENGTH Period Type $32;
   SET &RESULT;
   SELECT(PAF_TYPE);
     WHEN(1) TYPE='  Partial';
     WHEN(2) TYPE=' Cumulative';
     WHEN(3) TYPE='Cumulative (delta)';
     OTHERWISE;
   END;
   PERIOD=COMPRESS(TIME_1||"-<"||TIME_2);
   DROP TYPE PAF_TYPE TIME_:;
RUN;

%_EXIT_:

%MEND EST_PAF_M;
%*************************************************************************************************************;
