%*************************************************************************************************************;
%*                                                                                                           *;
%* Article: SAS macros for calculation of Population Attributable Fraction (PAF) in a cohort study design    *;
%* Authors: Maarit A. Laaksonen, Esa Virtala, Paul Knekt, Hannu Oja, and Tommi Hrknen                      *;
%*                                                                                                           *;
%*                                                                                                           *;
%* GEN_DATA                                                                                                  *;
%*                                                                                                           *;
%* Preparation of input data sets needed for the calculation of Population Attributable Fraction (PAF)       *;
%* for total mortality or disease incidence.                                                                 *;
%*                                                                                                           *;
%* INPUT:                                                                                                    *;
%*                                                                                                           *;
%*   INDATA                 = Original data set.                                                             *;
%*                                                                                                           *;
%*   IDVARIABLE             = Individual identification variable.                                            *;
%*                                                                                                           *;
%*   CENSORVARIABLE_1       = Variable which indicates whether censoring before the outcome of interest 1    *;
%*                            occurred. A binary variable (0/1).                                             *;
%*                                                                                                           *;
%*   CENSORVALUE_1          = Value (0/1) which indicates that censoring before the outcome of interest 1    *;
%*                            occurred.                                                                      *;
%*                                                                                                           *;
%*   FTVARIABLE_1           = Variable which indicates the individual length of follow-up until the outcome  *;
%*                            of interest 1 or censoring.                                                    *;
%*                                                                                                           *;
%*   CENSORVARIABLE_2       = Variable which indicates whether censoring before the outcome of interest 2    *;
%*                            occurred. A binary variable (0/1).                                             *;
%*                                                                                                           *;
%*   CENSORVALUE_2          = Value (0/1) which indicates that censoring before the outcome of interest 2    *;
%*                            occurred.                                                                      *;
%*                                                                                                           *;
%*   FTVARIABLE_2           = Variable which indicates the individual length of follow-up until the outcome  *;
%*                            of interest 2 or censoring.                                                    *;
%*                                                                                                           *;
%*   FTLENGTH               = Length of follow-up.                                                           *;
%*                                                                                                           *;
%*   PERIODLENGTH           = Length of follow-up intervals.                                                 *;
%*                                                                                                           *;
%*   DROPVARIABLES          = Variables to be dropped out from output data set.                              *;
%*                                                                                                           *;
%*   OUTDATA                = Output data set.                                                               *;
%*                                                                                                           *;
%* OUTPUT:                                                                                                   *;
%*                                                                                                           *;
%*   PERIODVARIABLE         = Categorical variable which indicates the cumulative follow-up time by the      *;
%*                            beginning of each follow-up time interval.                                     *;
%*                                                                                                           *;
%*   TIMEVARIABLE           = Variable which indicates the individual length of follow-up within each        *;
%*                            follow-up time interval.                                                       *;
%*                                                                                                           *;
%*************************************************************************************************************;

%MACRO GEN_DATA(
  INDATA           = ,
  IDVARIABLE       = ,
  CENSORVARIABLE_1 = ,
  CENSORVALUE_1    = 0,
  FTVARIABLE_1     = ,
  CENSORVARIABLE_2 = ,
  CENSORVALUE_2    = 0,
  FTVARIABLE_2     = ,
  FTLENGTH         = ,
  PERIODLENGTH     = ,
  DROPVARIABLES    = ,
  OUTDATA          = ,
  PERIODVARIABLE   = ,
  TIMEVARIABLE     =
);
  %IF &CENSORVALUE_1=%STR( ) %THEN %LET CENSORVALUE_1=0;
  %IF &CENSORVALUE_2=%STR( ) %THEN %LET CENSORVALUE_2=0;
  data &outdata(drop=&dropvariables _d _t);
    retain &idvariable &censorvariable_1 &timevariable &periodvariable;
    set &indata;
    by &idvariable;
    %IF &CENSORVARIABLE_2 NE %STR() %THEN %DO;
        if &censorvariable_2 ne &censorvalue_2 then do;
           &censorvariable_1 = &censorvalue_1;
           &ftvariable_1 = &ftvariable_2;
        end;
    %END;
    if first.&idvariable then do;
       &timevariable = &periodlength;
       _d = &censorvariable_1;
       _t = 0;
    end;
    do &periodvariable = 0 to &ftlength by &periodlength;
       &censorvariable_1 = &censorvalue_1;
       _t + &periodlength;
       if &ftvariable_1 < _t then do;
          &censorvariable_1 = _d;
          &timevariable = &ftvariable_1 - &periodvariable;
       end;
       if &timevariable > 0 then output;
    end;
    label
      &periodvariable="Followup period [&PERIODLENGTH]"
      &timevariable="Followup time [&FTLENGTH]";
  run;
%MEND GEN_DATA;
