%*************************************************************************************************************;
%*                                                                                                           *;
%* Article: SAS macros for calculation of Population Attributable Fraction (PAF) in a cohort study design    *;
%* Authors: Maarit A. Laaksonen, Esa Virtala, Paul Knekt, Hannu Oja, and Tommi Hrknen                      *;
%*                                                                                                           *;
%*                                                                                                           *;
%* PAF_D                                                                                                     *;
%*                                                                                                           *;
%* Calculation of relative risks (RR) and Population Attributable Fractions (PAF) of disease incidence and   *;
%* their confidence intervals for risk factors of interest.                                                  *;
%*                                                                                                           *;
%* INPUT:                                                                                                    *;
%*                                                                                                           *;
%*   DATASET_M              = Input data matrix for death.                                                   *;
%*                                                                                                           *;
%*   CENSORVARIABLE_M       = Variable which indicates whether censoring before death occurred. A binary     *;
%*                            variable (0/1).                                                                *;
%*                                                                                                           *;
%*   CENSORVALUE_M          = Value (0/1) which indicates that censoring before death occurred.              *;
%*                                                                                                           *;
%*   TIMEVARIABLE_M         = Variable which indicates the individual length of follow-up within each        *;
%*                            follow-up time interval.                                                       *;
%*                                                                                                           *;
%*   PERIODVARIABLE_M       = Categorical variable which indicates the cumulative follow-up time by the      *;
%*                            beginning of each follow-up time interval.                                     *;
%*                                                                                                           *;
%*   DATASET_D              = Input data matrix for disease.                                                 *;
%*                                                                                                           *;
%*   CENSORVARIABLE_D       = Variable which indicates whether censoring before incidence of disease         *;
%*                            occurred. A binary variable (0/1).                                             *;
%*                                                                                                           *;
%*   CENSORVALUE_D          = Value (0/1) which indicates that censoring before incidence of disease         *;
%*                            occurred.                                                                      *;
%*                                                                                                           *;
%*   TIMEVARIABLE_D         = Variable which indicates the individual length of follow-up within each        *;
%*                            follow-up time interval.                                                       *;
%*                                                                                                           *;
%*   PERIODVARIABLE_D       = Categorical variable which indicates the cumulative follow-up time by the      *;
%*                            beginning of each follow-up time interval.                                     *;
%*                                                                                                           *;
%*   IDVARIABLE             = Individual identification variable.                                            *;
%*                                                                                                           *;
%*   COHORTVARIABLE         = Categorical variable which indicates to which birth cohort the individual      *;
%*                            belongs to. Optional: if not given, the baseline hazard in the piecewise       *;
%*                            constant hazard model is only stratified according to the follow-up time       *;
%*                            interval.                                                                      *;
%*                                                                                                           *;
%*   DELTALENGTH            = Length of follow-up.                                                           *;
%*                                                                                                           *;
%*   CLASSVARIABLES         = Categorical variables included in the model (separated by blanks).             *;
%*                                                                                                           *;
%*   CLASSORDER             = Order of the categories of the variables in the lifereg analysis (affects      *;
%*                            the reference category): DESC=descending / ASC=ascending.                      *;
%*                                                                                                           *;
%*   COVARIATE_MODEL        = All the variables (categorical, continuous, or their interactions) included    *;
%*                            in the model (separated by blanks, interactions denoted by asterisk between    *;
%*                            the variables, VARIABLE1*VARIABLE2). Corresponds to the normal model statement *;
%*                            syntax of SAS.                                                                 *;
%*                                                                                                           *;
%*   GROUPVARIABLE          = The PAF estimates are calculated separately in the subgroups defined by this   *;
%*                            variable. Optional: if not given, the PAF estimate is calculated in the total  *;
%*                            sample (Note: if the GROUPVARIABLE is considered to be a potential effect      *;
%*                            modifying factor, an interaction term between the GROUPVARIABLE and the risk   *;
%*                            factor of interest must be included in the COVARIATE_MODEL).                   *;
%*                                                                                                           *;
%*   MODIFICATIONS          = Determines the reference category for the risk factors of interest to          *;
%*                            which all or some individuals are hypothetically moved in the calculation      *;
%*                            of PAF. If all the individuals from all the categories of the risk factor      *;
%*                            are moved to the reference category, the modification statement is denoted     *;
%*                            by 'RISK FACTOR = REFERENCE CATEGORY'. If individuals from only some of        *;
%*                            the categories of the risk factor are moved to the reference category,         *;
%*                            then the modification statement is denoted using the IF-THEN statement:        *;
%*                            'IF RISK FACTOR IN(CATEGORIES THAT ARE MODIFIED SEPARATED BY COMMA)            *;
%*                             THEN RISK FACTOR = REFERENCE CATEGORY'. If several risk factors are           *;
%*                            modified simultaneously, they are separated by semicolons.                     *;
%*                                                                                                           *;
%*   PRINT                  = Defines what is printed out from the LIFEREG procedure: YES = all output /     *;
%*                            NO = only the relative risks (RR) and their 95% confidence intervals (CI)      *;
%*                           (formed with the help of the PROC LIFEREG output). Optional: if not given,      *;
%*                            only the RRs and their 95% CIs are printed out.                                *;
%*                                                                                                           *;
%* OUTPUT:                                                                                                   *;
%*                                                                                                           *;
%*    - Relative risks of death and disease incidence and their 95% confidence intervals for risk factors    *;
%*      of interest. (a more comprehensive output from the LIFEREG procedure optional).                      *;
%*    - Cumulative PAFs for disease incidence, their standard errors and 95% confidence intervals for        *;
%*      chosen risk factor/s.                                                                                *;
%*                                                                                                           *;
%*************************************************************************************************************;

%MACRO PAF_D(
  DATASET_M        = ,
  CENSORVARIABLE_M = ,
  CENSORVALUE_M    = 0,
  TIMEVARIABLE_M   = ,
  PERIODVARIABLE_M = ,
  DATASET_D        = ,
  CENSORVARIABLE_D = ,
  CENSORVALUE_D    = 0,
  TIMEVARIABLE_D   = ,
  PERIODVARIABLE_D = ,
  IDVARIABLE       = ,
  COHORTVARIABLE   = ,
  DELTALENGTH      = ,
  CLASSVARIABLES   = ,
  CLASSORDER       = ,
  COVARIATE_MODEL  = ,
  GROUPVARIABLE    = ,
  MODIFICATIONS    = ,
  PRINT            = ,
);

  %LET _STARTTIME_=%SYSFUNC(TIME());

  %LET CENSORVARIABLE_M = %UPCASE(&CENSORVARIABLE_M);
  %LET CENSORVARIABLE_D = %UPCASE(&CENSORVARIABLE_D);
  %LET CENS_M = &CENSORVARIABLE_M;
  %LET CENS_D = &CENSORVARIABLE_D;
  %IF %LENGTH(&CENS_M)>20 %THEN %LET CENS_M = %SUBSTR(&CENSORVARIABLE_M,1,20);
  %IF %LENGTH(&CENS_D)>20 %THEN %LET CENS_D = %SUBSTR(&CENSORVARIABLE_D,1,20);
  %IF &CENSORVALUE_M=%STR( ) %THEN %LET CENSORVALUE_M=0;
  %IF &CENSORVALUE_D=%STR( ) %THEN %LET CENSORVALUE_D=0;

  %LET CLASSORDER = %UPCASE(&CLASSORDER);
  %IF &CLASSORDER^=DESC %THEN %LET CLASSORDER = ASC;

  %LET GROUPVARIABLE = %UPCASE(&GROUPVARIABLE);

  %IF %UPCASE(&PRINT)^=YES %THEN %LET PRINT=NO;

  %* Clean work *;
  %LET DSID1=%SYSFUNC(OPEN(&DATASET_M));
  %LET LIB1=;
  %LET MEM1=;
  %LET DSID2=%SYSFUNC(OPEN(&DATASET_D));
  %LET LIB2=;
  %LET MEM2=;
  %LET MEM=;
  %IF &DSID1 AND &DSID2 %THEN %DO;
      %LET LIB1=%SYSFUNC(ATTRC(&DSID1,LIB));
      %LET MEM1=%SYSFUNC(ATTRC(&DSID1,MEM));
      %LET DSID1=%SYSFUNC(CLOSE(&DSID1));
      %LET LIB2=%SYSFUNC(ATTRC(&DSID2,LIB));
      %LET MEM2=%SYSFUNC(ATTRC(&DSID2,MEM));
      %LET DSID2=%SYSFUNC(CLOSE(&DSID2));
  %END;
  %IF &LIB1=WORK %THEN %LET MEM=&MEM1;
  %IF &LIB2=WORK %THEN %LET MEM=&MEM &MEM2;
  PROC DATASETS LIBRARY=WORK MEMTYPE=DATA NOLIST
     %IF &MEM NE %STR() %THEN %STR( ;SAVE &MEM; );
     %ELSE %STR( KILL; );
     RUN;
  QUIT;

   %* Calculate periodlength and number of periods *;
  PROC SQL NOPRINT;
    SELECT MAX(&TIMEVARIABLE_M) INTO :PL_M FROM &DATASET_M;
    SELECT MAX(&TIMEVARIABLE_D) INTO :PL_D FROM &DATASET_D;
    SELECT COUNT(DISTINCT &PERIODVARIABLE_M) INTO :PC_M FROM &DATASET_M;
    SELECT COUNT(DISTINCT &PERIODVARIABLE_D) INTO :PC_D FROM &DATASET_D;
  QUIT;
  %IF &SYSERR %THEN %GOTO _OOPS_;
  %LET PERIODLENGTH=%SYSFUNC(MAX(&PL_M,&PL_D));
  %LET PERIODCOUNT=%SYSFUNC(MAX(&PC_M,&PC_D));

  %* If deltalength is missing, take maximum of observed follow-up time *;
  %IF &DELTALENGTH=%STR( ) %THEN %DO;
      PROC SQL NOPRINT;
        SELECT CEIL(MAX(TIMES)) INTO :DL_M FROM
          (SELECT SUM(&TIMEVARIABLE_M) AS TIMES FROM &DATASET_M GROUP BY &IDVARIABLE);
        SELECT CEIL(MAX(TIMES)) INTO :DL_D FROM
          (SELECT SUM(&TIMEVARIABLE_D) AS TIMES FROM &DATASET_D GROUP BY &IDVARIABLE);
      QUIT;
      %LET DELTALENGTH=%SYSFUNC(MAX(&DL_M,&DL_D));
  %END;

  %* Call EST_MATRIX macro for preparation of design matrices, parameter estimates and their covariances *;
  %* needed for the estimation of Population Attributable Fraction (PAF) and its confidence interval     *;
  %* separately for death and disease                                                                    *;
  %* (see more details about function of this macro in EST_MATRIX.SAS)                                   *;

  %EST_MATRIX(
    DATASET         = &DATASET_M,
    CENSORVARIABLE  = &CENS_M,
    CENSORVALUE     = &CENSORVALUE_M,
    TIMEVARIABLE    = &TIMEVARIABLE_M,
    IDVARIABLE      = &IDVARIABLE,
    COHORTVARIABLE  = &COHORTVARIABLE,
    PERIODVARIABLE  = &PERIODVARIABLE_M,
    PERIODLENGTH    = &PERIODLENGTH,
    PERIODCOUNT     = &PERIODCOUNT,
    DELTALENGTH     = &DELTALENGTH,
    CLASSVARIABLES  = &CLASSVARIABLES,
    CLASSORDER      = &CLASSORDER,
    COVARIATE_MODEL = &COVARIATE_MODEL,
    MODIFICATIONS   = &MODIFICATIONS,
    PRINT           = &PRINT
  );

  ODS PDF STARTPAGE=NOW;

  %EST_MATRIX(
    DATASET         = &DATASET_D,
    CENSORVARIABLE  = &CENS_D,
    CENSORVALUE     = &CENSORVALUE_D,
    TIMEVARIABLE    = &TIMEVARIABLE_D,
    IDVARIABLE      = &IDVARIABLE,
    COHORTVARIABLE  = &COHORTVARIABLE,
    PERIODVARIABLE  = &PERIODVARIABLE_D,
    PERIODLENGTH    = &PERIODLENGTH,
    PERIODCOUNT     = &PERIODCOUNT,
    DELTALENGTH     = &DELTALENGTH,
    CLASSVARIABLES  = &CLASSVARIABLES,
    CLASSORDER      = &CLASSORDER,
    COVARIATE_MODEL = &COVARIATE_MODEL,
    MODIFICATIONS   = &MODIFICATIONS,
    PRINT           = &PRINT
  );

  %* Call EST_PAF_D macro for calculation of Population Attributable Fraction (PAF), its standard error *;
  %* and 95% confidence interval for disease incidence                                                  *;
  %* (see more details about function of this macro in EST_PAF_D.SAS)                                   *;
  %EST_PAF_D(
    DES      = DESIGN_1,
    DES_STAR = DESIGN_2,
    EST_M    = &CENS_M._EST,
    EST_D    = &CENS_D._EST,
    COVB_M   = &CENS_M._COVB,
    COVB_D   = &CENS_D._COVB,
    PERIODLENGTH = &PERIODLENGTH,
    PERIODCOUNT  = &PERIODCOUNT,
    DELTALENGTH  = &DELTALENGTH,
    RESULT = _PAF_D_M
  );
  DATA _PAF_D_M;
    LENGTH Censor $ 32;
    SET _PAF_D_M;
    CENSOR="&CENSORVARIABLE_D (&CENSORVARIABLE_M censoring)";
  RUN;

  ODS PDF STARTPAGE=NOW;
  PROC PRINT SPLIT="_" DATA=&CENS_M._RR(DROP=DF--PROBCHISQ)
    STYLE(TABLE)={PRETEXT="Relative risks of death"};
    FORMAT _NUMERIC_ 8.4;
    ID CENSOR PARAMETER;
  RUN;
  PROC PRINT SPLIT="_" DATA=&CENS_M._CONV LABEL NOOBS;
    LABEL REASON="! ! !";
    VAR REASON / STYLE={FONT_WEIGHT=BOLD};
  RUN;

  PROC PRINT SPLIT="_" DATA=&CENS_D._RR(DROP=DF--PROBCHISQ)
    STYLE(TABLE)={PRETEXT="Relative risks of incidence of disease"};
    FORMAT _NUMERIC_ 8.4;
    ID CENSOR PARAMETER;
  RUN;
  PROC PRINT SPLIT="_" DATA=&CENS_D._CONV LABEL NOOBS;
    LABEL REASON="! ! !";
    VAR REASON / STYLE={FONT_WEIGHT=BOLD};
  RUN;

  PROC PRINT SPLIT="_" DATA=_PAF_D_M
    STYLE(TABLE)={PRETEXT="PAF for disease incidence"};
    ID CENSOR PERIOD;
  RUN;

  %* If the GROUPVARIABLE is given in the PAF_D macro call, the point estimate, standard error and confidence interval *;
  %* of PAF for disease incidence are calculated separately in the subgroups defined by this variable                  *;

  %IF &GROUPVARIABLE^=%STR( ) %THEN %DO;

      PROC SQL NOPRINT;
        SELECT DISTINCT &GROUPVARIABLE INTO :GROUPVALUES SEPARATED BY '*'
          FROM &DATASET_M
         WHERE &GROUPVARIABLE IS NOT NULL;
        SELECT COUNT(DISTINCT &GROUPVARIABLE) INTO :GROUPCOUNT
          FROM &DATASET_M
         WHERE &GROUPVARIABLE IS NOT NULL;
      QUIT;

      %* GROUPWISE *;

      DATA _PAF_GROUP_D_M;
        LENGTH Censor $ 32 Group $ 64;
        SET _NULL_;
      RUN;

      %DO _I=1 %TO &GROUPCOUNT;
          %LET GROUPVALUE=%SCAN(&GROUPVALUES,&_I,%STR(*));

          %LET KEEP=1;
          PROC SQL NOPRINT;
            SELECT NAME INTO :KEEP SEPARATED BY " OR "
              FROM SASHELP.VCOLUMN
             WHERE LIBNAME="WORK" AND MEMNAME="DESIGN_1" AND INDEX(UPCASE(NAME),UPCASE("&GROUPVARIABLE.&GROUPVALUE"));
          QUIT;

          DATA _DES1_&GROUPVALUE;
            SET DESIGN_1;
            IF &KEEP;
          RUN;
          DATA _DES2_&GROUPVALUE;
            SET DESIGN_2;
            IF &KEEP;
          RUN;

          %EST_PAF_D(
            GROUP    = &GROUPVALUE,
            DES      = _DES1_&GROUPVALUE,
            DES_STAR = _DES2_&GROUPVALUE,
            EST_M    = &CENS_M._EST,
            EST_D    = &CENS_D._EST,
            COVB_M   = &CENS_M._COVB,
            COVB_D   = &CENS_D._COVB,
            PERIODLENGTH = &PERIODLENGTH,
            PERIODCOUNT  = &PERIODCOUNT,
            DELTALENGTH  = &DELTALENGTH,
            RESULT = _PAF_D_M_&GROUPVALUE
          );
          DATA _PAF_GROUP_D_M;
            SET _PAF_GROUP_D_M  _PAF_D_M_&GROUPVALUE(IN=NEW);
            CENSOR="&CENSORVARIABLE_D (&CENSORVARIABLE_M censoring)";
            IF NEW THEN GROUP="&GROUPVARIABLE = &GROUPVALUE";
          RUN;

      %END; %* OF GROUPCOUNT *;

      PROC PRINT SPLIT="_" DATA=_PAF_GROUP_D_M
        STYLE(TABLE)={PRETEXT="PAF for disease incidence in subgroups of &GROUPVARIABLE"};
        FORMAT _NUMERIC_ 8.4;
        ID CENSOR GROUP PERIOD;
      RUN;

      %* GROUP DIFFERENCES *;

      DATA _PAF_DIFF_D_M;
        LENGTH Censor $ 32 Group $ 64 Values $ 32;
        SET _NULL_;
      RUN;

      %DO _I=1 %TO %EVAL(&GROUPCOUNT-1);
          %LET GROUPVALUE1=%SCAN(&GROUPVALUES,&_I,%STR(*));
          %DO _J=%EVAL(&_I+1) %TO &GROUPCOUNT;
             %LET GROUPVALUE2=%SCAN(&GROUPVALUES,&_J,%STR(*));

              %* Call EST_PAF_DIFF_D macro for calculation of the difference between the groupwise PAF estimates, *;
              %* its 95% confidence intervals and a p-value for statistical significance of this difference       *;
              %* to determine the potential groupwise differences                                                 *;
              %* (see more details about function of this macro in EST_PAF_DIFF_D.SAS)                            *;
              %EST_PAF_DIFF_D(
                PAF_G1          = PAF_&GROUPVALUE1,
                PAF_G2          = PAF_&GROUPVALUE2,
                DPAF_G1_GAMMA_M = dPAF_&GROUPVALUE1._GAMMA_M,
                DPAF_G1_GAMMA_D = dPAF_&GROUPVALUE1._GAMMA_D,
                DPAF_G2_GAMMA_M = dPAF_&GROUPVALUE2._GAMMA_M,
                DPAF_G2_GAMMA_D = dPAF_&GROUPVALUE2._GAMMA_D,
                COVB_M          = &CENS_M._COVB,
                COVB_D          = &CENS_D._COVB,
                DELTALENGTH     = &DELTALENGTH,
                RESULT = _PAF_DIFF_D_M_&GROUPVALUE1._&GROUPVALUE2
              );
              DATA _PAF_DIFF_D_M;
                SET _PAF_DIFF_D_M  _PAF_DIFF_D_M_&GROUPVALUE1._&GROUPVALUE2(IN=NEW);
                CENSOR="&CENSORVARIABLE_D (&CENSORVARIABLE_M censoring)";
                GROUP="&GROUPVARIABLE";
                IF NEW THEN VALUES="&GROUPVALUE1 - &GROUPVALUE2";
              RUN;

          %END; %* OF GROUPVALUE 2 *;
      %END; %* OF GROUPVALUE 1 *;

      PROC PRINT SPLIT="_" DATA=_PAF_DIFF_D_M
        STYLE(TABLE)={PRETEXT="Statistical significance of differences between groupwise PAF estimates"};
        FORMAT _NUMERIC_ 8.4;
        ID CENSOR GROUP VALUES PERIOD;
      RUN;

  %END; %* OF GROUPVARIABLE *;

%_OOPS_:

  %LET _ENDTIME_=%SYSFUNC(TIME());
  %PUT %STR(        );
  %PUT %STR(WARNING-******************************);
  %PUT      WARNING-   Real time used: %SYSFUNC(SUM(&_ENDTIME_,-&_STARTTIME_),TOD8.);
  %PUT %STR(WARNING-******************************);
  %PUT %STR(        );

%MEND PAF_D;
%*************************************************************************************************************;
