%*************************************************************************************************************;
%*                                                                                                           *;
%* Article: SAS macros for calculation of Population Attributable Fraction (PAF) in a cohort study design    *;
%* Authors: Maarit A. Laaksonen, Esa Virtala, Paul Knekt, Hannu Oja, and Tommi Hrknen                      *;
%*                                                                                                           *;
%*                                                                                                           *;
%* PAF_EXAMPLE_D                                                                                             *;
%*                                                                                                           *;
%* Demonstration of calculation of relative risks (RR) and Population Attributable Fractions (PAF) of        *;
%* disease incidence and their confidence intervals for risk factors of interest.                            *;
%*                                                                                                           *;
%*************************************************************************************************************;

OPTIONS NOMPRINT NONOTES;

/*
  %* Program directory (Unix) *;
  %LET LIB=%STR(/PAF/);
*/

%* Program directory (Windows) *;
%LET LIB=%STR(C:\PAF\);

%LET LIB=%LOWCASE(&LIB);

LIBNAME X "&LIB";

%INCLUDE "&lib.gen_data.sas" "&lib.est_*.sas" "&lib.paf_d.sas";

%* Data preparation *;

%* 5-year follow-up time intervals *;

%* Input data matrix for death *;
%GEN_DATA(
  INDATA           = X.EXAMPLE,
  IDVARIABLE       = ID,
  CENSORVARIABLE_1 = DEATH,
  CENSORVALUE_1    = 0,
  FTVARIABLE_1     = DEATH_FT,
  CENSORVARIABLE_2 = DIAB,
  CENSORVALUE_2    = 0,
  FTVARIABLE_2     = DIAB_FT,
  FTLENGTH         = 20,
  PERIODLENGTH     = 5,
  DROPVARIABLES    = DIAB DIAB_FT DEATH_FT,
  OUTDATA          = X.EXAMPLE_DEATH_2,
  PERIODVARIABLE   = F_PERIOD,
  TIMEVARIABLE     = F_TIME
);

%* Input data matrix for disease *;
%GEN_DATA(
  INDATA           = X.EXAMPLE,
  IDVARIABLE       = ID,
  CENSORVARIABLE_1 = DIAB,
  CENSORVALUE_1    = 0,
  FTVARIABLE_1     = DIAB_FT,
  CENSORVARIABLE_2 = ,
  CENSORVALUE_2    = ,
  FTVARIABLE_2     = ,
  FTLENGTH         = 20,
  PERIODLENGTH     = 5,
  DROPVARIABLES    = DEATH DEATH_FT DIAB_FT,
  OUTDATA          = X.EXAMPLE_DISEASE,
  PERIODVARIABLE   = F_PERIOD,
  TIMEVARIABLE     = F_TIME
);

%* Examples of macro calls for calculation of PAF for disease incidence *;

%* Follow-up of 17 years *;
%* Adjustment for sex and age (through birth cohort) *;

%* Risk factor of interest: smoking *;
ODS LISTING CLOSE;
ODS NOPROCTITLE;
ODS PDF FILE="&lib.paf_example_d_smoke.pdf" STYLE=NORMAL STARTPAGE=OFF;

TITLE "PAF for Disease, modification of smoking";
%PAF_D(
  DATASET_M        = X.EXAMPLE_DEATH_2,
  CENSORVARIABLE_M = DEATH,
  CENSORVALUE_M    = 0,
  TIMEVARIABLE_M   = F_TIME,
  PERIODVARIABLE_M = F_PERIOD,
  DATASET_D        = X.EXAMPLE_DISEASE,
  CENSORVARIABLE_D = DIAB,
  CENSORVALUE_D    = 0,
  TIMEVARIABLE_D   = F_TIME,
  PERIODVARIABLE_D = F_PERIOD,
  IDVARIABLE       = ID,
  COHORTVARIABLE   = B_COHORT,
  DELTALENGTH      = 17,
  CLASSVARIABLES   = SEX SMOKE,
  CLASSORDER       = DESC,
  COVARIATE_MODEL  = SEX SMOKE,
  GROUPVARIABLE    = ,
  MODIFICATIONS    = IF SMOKE IN(3,4) THEN SMOKE=1,
  PRINT            =
);
TITLE;
ODS PDF CLOSE;
ODS LISTING;
RUN;

%* Risk factor of interest: body mass index *;
ODS LISTING CLOSE;
ODS NOPROCTITLE;
ODS PDF FILE="&lib.paf_example_d_bmi.pdf" STYLE=NORMAL STARTPAGE=OFF;

TITLE "PAF for Disease, modification of BMI";
%PAF_D(
  DATASET_M        = X.EXAMPLE_DEATH_2,
  CENSORVARIABLE_M = DEATH,
  CENSORVALUE_M    = 0,
  TIMEVARIABLE_M   = F_TIME,
  PERIODVARIABLE_M = F_PERIOD,
  DATASET_D        = X.EXAMPLE_DISEASE,
  CENSORVARIABLE_D = DIAB,
  CENSORVALUE_D    = 0,
  TIMEVARIABLE_D   = F_TIME,
  PERIODVARIABLE_D = F_PERIOD,
  IDVARIABLE       = ID,
  COHORTVARIABLE   = B_COHORT,
  DELTALENGTH      = 17,
  CLASSVARIABLES   = SEX BMI_2,
  CLASSORDER       = DESC,
  COVARIATE_MODEL  = SEX BMI_2,
  GROUPVARIABLE    = ,
  MODIFICATIONS    = BMI_2=1,
  PRINT            =
);
TITLE;
ODS PDF CLOSE;
ODS LISTING;
RUN;

%* Risk factor of interest: blood pressure *;
ODS LISTING CLOSE;
ODS NOPROCTITLE;
ODS PDF FILE="&lib.paf_example_d_bp.pdf" STYLE=NORMAL STARTPAGE=OFF;

TITLE "PAF for Disease, modification of blood pressure";
%PAF_D(
  DATASET_M        = X.EXAMPLE_DEATH_2,
  CENSORVARIABLE_M = DEATH,
  CENSORVALUE_M    = 0,
  TIMEVARIABLE_M   = F_TIME,
  PERIODVARIABLE_M = F_PERIOD,
  DATASET_D        = X.EXAMPLE_DISEASE,
  CENSORVARIABLE_D = DIAB,
  CENSORVALUE_D    = 0,
  TIMEVARIABLE_D   = F_TIME,
  PERIODVARIABLE_D = F_PERIOD,
  IDVARIABLE       = ID,
  COHORTVARIABLE   = B_COHORT,
  DELTALENGTH      = 17,
  CLASSVARIABLES   = SEX BP,
  CLASSORDER       = DESC,
  COVARIATE_MODEL  = SEX BP,
  GROUPVARIABLE    = ,
  MODIFICATIONS    = BP=1,
  PRINT            =
);
TITLE;
ODS PDF CLOSE;
ODS LISTING;
RUN;

%* Risk factor of interest: BMI                      *;
%* Potential effect modifying factor: Blood pressure *;
ODS LISTING CLOSE;
ODS NOPROCTITLE;
ODS PDF FILE="&lib.paf_example_d_bp_x_bmi.pdf" STYLE=NORMAL STARTPAGE=OFF;

TITLE "PAF for Disease, modification of BMI in subgroups defined by blood pressure";
%PAF_D(
  DATASET_M        = X.EXAMPLE_DEATH_2,
  CENSORVARIABLE_M = DEATH,
  CENSORVALUE_M    = 0,
  TIMEVARIABLE_M   = F_TIME,
  PERIODVARIABLE_M = F_PERIOD,
  DATASET_D        = X.EXAMPLE_DISEASE,
  CENSORVARIABLE_D = DIAB,
  CENSORVALUE_D    = 0,
  TIMEVARIABLE_D   = F_TIME,
  PERIODVARIABLE_D = F_PERIOD,
  IDVARIABLE       = ID,
  COHORTVARIABLE   = B_COHORT,
  DELTALENGTH      = 17,
  CLASSVARIABLES   = SEX BP BMI_2,
  CLASSORDER       = DESC,
  COVARIATE_MODEL  = SEX BP BP*BMI_2,
  GROUPVARIABLE    = BP,
  MODIFICATIONS    = BMI_2=1,
  PRINT            =
);
TITLE;
ODS PDF CLOSE;
ODS LISTING;
OPTIONS NOTES;
RUN;
